package ids;

/*
 * reading and writing signatures to the specified file
 * rewritten by jk 2003.12.06
 */

import java.io.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import javax.swing.*;
import org.w3c.dom.*;
import org.xml.sax.*;


public class Parse {
    File file = null;
    static Document document = null;
    static final String JAXP_SCHEMA_LANGUAGE =
        "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
    static final String schemaSource = ids.Translator.commonXSD;
    static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";

    public Parse(File f) {
        file = new File(f.getAbsolutePath());
        parseFile(file);
    }

    public Parse() {
        System.out.println("Constructing data base");
        file = new File(OptionsPane.getDataBaseFile());
        parseFile(file);
    }



    public Document parseFile(File file) { // read xsd file and return DOM document

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setAttribute(JAXP_SCHEMA_SOURCE, new File(schemaSource));

        factory.setValidating(true);
        factory.setNamespaceAware(true);
        factory.setIgnoringElementContentWhitespace(true);

        try {
            factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
        } catch (IllegalArgumentException x) {
            System.out.println("Parser do not know this XML schema!");
            return null;
            /* return added - jk 2004.03.17 */
        }


	DocumentBuilder builder = null;
        try	{
            builder = factory.newDocumentBuilder();
            builder.setErrorHandler(
                new org.xml.sax.ErrorHandler(){

                    public void fatalError(SAXParseException e)
                    throws SAXException {
                        System.out.println("**********PARSER ERROR!**********");
                        throw e;
                    }
                    public void error(SAXParseException e)
                    throws SAXParseException {
                        System.out.println("**********PARSER ERROR!**********");
                        throw e;
                    }
                    public void warning(SAXParseException err)
                    throws SAXParseException {
                        System.out.println("** Warning"
                                           + ", line " + err.getLineNumber()
                                           + ", uri " + err.getSystemId());
                        System.out.println("   " + err.getMessage());
                    }
                }
            );

            builder.isValidating();

            System.out.println("building document: " + file.getName());
            File sigFile = new File(file.getAbsolutePath());
            if (sigFile.length() <= 0) {
              System.out.print("File "+file.getName()+" is empty: ");
              createEmptyDocument();
              return (document);
            }
            document = builder.parse(sigFile);


//            DOMConfiguration docConfig = document.getDomConfig();
//            docConfig.setParameter("normalize-characters", true);
//              docConfig.setParameter("element-content-whitespace", false);

            document.getDocumentElement().normalize();
            document.normalize();
        } catch(SAXParseException spe)	{
            System.out.println ("\n** Parsing error"
                +	", line	" +	spe.getLineNumber ()
                +	", uri " + spe.getSystemId ());
            System.out.println("	  "	+ spe.getMessage() );

            if (spe.getException() != null)
                spe.getException().printStackTrace();
            else
                spe.printStackTrace();
        } catch	(SAXException sxe) {
            if (sxe.getException() != null)
               sxe.getException().printStackTrace();
            else
                sxe.printStackTrace();
        } catch	(ParserConfigurationException pce) {
            pce.printStackTrace();
        } catch	(IOException ioe) {
            JOptionPane.showMessageDialog(
                null, "File opening FAILED", "WARNING",
        	JOptionPane.WARNING_MESSAGE);
	    try {
			createEmptyDocument();
	    } catch (Exception e) {
        	JOptionPane.showMessageDialog(
            	    null, "Error creating empty document", "WARNING",
                    JOptionPane.WARNING_MESSAGE);
		return null;
	    }
        }

        DomToTreeModelAdapter dttma = new DomToTreeModelAdapter();
        JTree tree = new JTree(dttma);
        tree.getSelectionPath();
		DataGui.setTree(tree);

       return document;
    }

    static void saveToFile(File f){
        try {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
//            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount","2");
            DOMSource source = new DOMSource(document);
            StreamResult result = new StreamResult(f);
            transformer.transform(source, result);
            JOptionPane.showMessageDialog(null, "Saved succesfull");
            return;
        }
        catch (TransformerConfigurationException tce) {
            // Error generated by the parser
            System.out.println("* Transformer Factory error\n " + tce.getMessage());
            // Use the contained exception, if any
            if (tce.getException() != null)
                tce.getException().printStackTrace();
            else
                tce.printStackTrace();
        }
        catch (TransformerException te) {
            // Error generated by the parser
            System.out.println("* Transformation error\n "+ te.getMessage());
            // Use the contained exception, if any
            if (te.getException() != null)
                te.getException().printStackTrace();
            else
                te.printStackTrace();
        }
        catch (Exception e) {
            System.out.println("Error");
            System.out.println(e.toString());
            System.out.println(e.getClass().toString());
            e.printStackTrace();
        }
    }


    public static void setDocument(Document d) {
		document = d;
    }
    public static void createEmptyDocument() {
    	System.out.println("Creating empty document...");
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setAttribute(JAXP_SCHEMA_SOURCE, new File(schemaSource));
		DocumentBuilder builder = null;
		try {
        	builder = factory.newDocumentBuilder();
    	} catch (ParserConfigurationException pce) {
            pce.printStackTrace();
    	}
        document = builder.newDocument();
        org.w3c.dom.Element sig = document.createElement("Signatures");
        sig.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
        sig.setAttribute("xsi:noNamespaceSchemaLocation", ids.Translator.commonXSD);
        org.w3c.dom.Node node = null;
		document.appendChild(sig);;
        DomToTreeModelAdapter dttma = new DomToTreeModelAdapter();
        JTree tree = new JTree(dttma);
        tree.getSelectionPath();
		DataGui.setTree(tree);

    }
}

